@@ -16,7 +16,7 @@ from jax import (
 import jax.numpy as jnp
 
 from numpyro import handlers
-from numpyro.contrib.control_flow.util import PytreeTrace
+from numpyro.ops.pytree import PytreeTrace
 from numpyro.primitives import _PYRO_STACK, Messenger, apply_stack
 from numpyro.util import not_jax_tracer
 
@@ -378,19 +378,19 @@ def scan(f, init, xs, length=None, reverse=False, history=1):
         variables will contain the following sites:
 
             + init sites: those sites belong to the first `history` traces of `f`.
-                Sites at the `i`-th trace will have name prefixed with
-                `'_PREV_' * (2 * history - 1 - i)`.
+              Sites at the `i`-th trace will have name prefixed with
+              `'_PREV_' * (2 * history - 1 - i)`.
             + scanned sites: those sites collect the values of the remaining scan
-                loop over `f`. An addition time dimension `_time_foo` will be
-                added to those sites, where `foo` is the name of the first site
-                appeared in `f`.
+              loop over `f`. An addition time dimension `_time_foo` will be
+              added to those sites, where `foo` is the name of the first site
+              appeared in `f`.
 
         Not all transition functions `f` are supported. All of the restrictions from
         Pyro's enumeration tutorial [2] still apply here. In addition, there should
         not have any site outside of `scan` depend on the first output of `scan`
         (the last carry value).
 
-    ** References **
+    **References**
 
     1. *Temporal Parallelization of Bayesian Smoothers*,
        Simo Sarkka, Angel F. Garcia-Fernandez
@@ -436,12 +436,16 @@ def scan(f, init, xs, length=None, reverse=False, history=1):
 
     if not msg["kwargs"].get("enum", False):
         for msg in pytree_trace.trace.values():
+            if msg["type"] == "plate":
+                continue
             apply_stack(msg)
     else:
         from numpyro.contrib.funsor import to_funsor
         from numpyro.contrib.funsor.enum_messenger import LocalNamedMessenger
 
         for msg in pytree_trace.trace.values():
+            if msg["type"] == "plate":
+                continue
             with LocalNamedMessenger():
                 dim_to_name = msg["infer"].get("dim_to_name")
                 to_funsor(
