@@ -834,7 +834,7 @@ class AutoDAIS(AutoContinuous):
             p_grad = beta * grad(log_density)(z_half)
             v_hat = v_prev + eta * (q_grad + p_grad)
             z = z_half + v_hat * eta * inv_mass_matrix
-            v = gamma * v_hat + jnp.sqrt(1 - gamma ** 2) * eps
+            v = gamma * v_hat + jnp.sqrt(1 - gamma**2) * eps
             delta_ke = momentum_dist.log_prob(v_prev) - momentum_dist.log_prob(v_hat)
             log_factor = log_factor + delta_ke
             return (z, v, log_factor), None
@@ -885,16 +885,7 @@ class AutoDiagonalNormal(AutoContinuous):
         prefix="auto",
         init_loc_fn=init_to_uniform,
         init_scale=0.1,
-        init_strategy=None,
     ):
-        if init_strategy is not None:
-            init_loc_fn = init_strategy
-            warnings.warn(
-                "`init_strategy` argument has been deprecated in favor of `init_loc_fn`"
-                " argument.",
-                FutureWarning,
-                stacklevel=find_stack_level(),
-            )
         if init_scale <= 0:
             raise ValueError("Expected init_scale > 0. but got {}".format(init_scale))
         self._init_scale = init_scale
@@ -955,16 +946,7 @@ class AutoMultivariateNormal(AutoContinuous):
         prefix="auto",
         init_loc_fn=init_to_uniform,
         init_scale=0.1,
-        init_strategy=None,
     ):
-        if init_strategy is not None:
-            init_loc_fn = init_strategy
-            warnings.warn(
-                "`init_strategy` argument has been deprecated in favor of `init_loc_fn`"
-                " argument.",
-                FutureWarning,
-                stacklevel=find_stack_level(),
-            )
         if init_scale <= 0:
             raise ValueError("Expected init_scale > 0. but got {}".format(init_scale))
         self._init_scale = init_scale
@@ -1029,16 +1011,7 @@ class AutoLowRankMultivariateNormal(AutoContinuous):
         init_loc_fn=init_to_uniform,
         init_scale=0.1,
         rank=None,
-        init_strategy=None,
     ):
-        if init_strategy is not None:
-            init_loc_fn = init_strategy
-            warnings.warn(
-                "`init_strategy` argument has been deprecated in favor of `init_loc_fn`"
-                " argument.",
-                FutureWarning,
-                stacklevel=find_stack_level(),
-            )
         if init_scale <= 0:
             raise ValueError("Expected init_scale > 0. but got {}".format(init_scale))
         self._init_scale = init_scale
@@ -1048,7 +1021,7 @@ class AutoLowRankMultivariateNormal(AutoContinuous):
         )
 
     def _get_posterior(self, *args, **kwargs):
-        rank = int(round(self.latent_dim ** 0.5)) if self.rank is None else self.rank
+        rank = int(round(self.latent_dim**0.5)) if self.rank is None else self.rank
         loc = numpyro.param("{}_loc".format(self.prefix), self._init_latent)
         cov_factor = numpyro.param(
             "{}_cov_factor".format(self.prefix), jnp.zeros((self.latent_dim, rank))
@@ -1232,16 +1205,7 @@ class AutoIAFNormal(AutoContinuous):
         hidden_dims=None,
         skip_connections=False,
         nonlinearity=stax.Elu,
-        init_strategy=None,
     ):
-        if init_strategy is not None:
-            init_loc_fn = init_strategy
-            warnings.warn(
-                "`init_strategy` argument has been deprecated in favor of `init_loc_fn`"
-                " argument.",
-                FutureWarning,
-                stacklevel=find_stack_level(),
-            )
         self.num_flows = num_flows
         # 2-layer, stax.Elu, skip_connections=False by default following the experiments in
         # IAF paper (https://arxiv.org/abs/1606.04934)
@@ -1319,16 +1283,7 @@ class AutoBNAFNormal(AutoContinuous):
         init_loc_fn=init_to_uniform,
         num_flows=1,
         hidden_factors=[8, 8],
-        init_strategy=None,
     ):
-        if init_strategy is not None:
-            init_loc_fn = init_strategy
-            warnings.warn(
-                "`init_strategy` argument has been deprecated in favor of `init_loc_fn`"
-                " argument.",
-                FutureWarning,
-                stacklevel=find_stack_level(),
-            )
         self.num_flows = num_flows
         self._hidden_factors = hidden_factors
         super(AutoBNAFNormal, self).__init__(
